<?php

namespace Fir\Controllers;

use Aws\S3\S3Client;

class Admin extends Controller
{
    /**
     * This would be your http://localhost/project-name/ index page
     *
     * @return array
     */
    protected $admin;

    public function index() {
        if (isset($this->url[1]) && $this->url[1] == 'lang') {
            $this->updateLanguage($this->url[2]);
        }
        redirect('admin/login');
    }
	
    public function login()
    {

        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
			
		$validation = "";	

		/* Use Admin Model */
        $admin = $this->library('Admin');
		if($admin->isLoggedIn() === true):
		 redirect('admin/dashboard');
		endif;
		
		/* Use Settings Model */
		$settingsModel = $this->model('Settings');
        $data['settings'] = $settingsModel->get();

        // If the user tries to log-in
			if(isset($_POST['login'])) {
		
			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
			  'email' => [
				 'required' => true,
				 'minlength' => 2,
				 'maxlength' => 200
			   ],
			  'password' => [
				 'required' => true,
				 'minlength' => 2,
				 'maxlength' => 200
			   ]
			]);
				 
			if (!$validation->fails()) {
					$email = $_POST['email'];
					$password = $_POST['password'];
					$remember = null;
					
					if(isset($_POST['remember'])) {
					  $remember = ($_POST['remember'] === 'on') ? true : false;
					}

					// Attempt to auth the user
					$auth = $admin->login(
						 $email,
						 $password,
						 $remember
					  );

					// If the user has been logged-in
					if($auth) {
						redirect('admin/dashboard');
					}
					// If the user could not be logged-in
					elseif(isset($_POST['login'])) {
						$_SESSION['message'][] = ['error', $this->lang['invalid_user_pass']];
					}

				}
			else {
			 foreach ($validation->errors()->all() as $err) {
				$str = implode(" ",$err);
				 foreach ($err as $r) {
					$_SESSION['errors'][] = ['error', $r];
				 }	
			 }
			}
		}	

        return ['content' => $this->view->render($data, 'admin/login')];
    }

    public function dashboard() {
        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Product Model */
        $productModel = $this->model('Product');
		$data['count_products'] = $productModel->cproducts();
		$data['count_freebies'] = $productModel->cfreebies();
		
		$downloadsModel = $this->model('Downloads');	
        $data['total_downloads'] = $downloadsModel->countD();
		
		/* Use Transaction Model */
        $transactionsModel = $this->model('Transactions');
		$data['payments'] = $transactionsModel->payments();
		
		/* Use User Model */
        $userModel = $this->model('User');
		$data['users'] = $userModel->cusers();
		$data['verified_users'] = $userModel->vusers();
		
		$data['themes'] = $this->getThemes();
		
		/* Use Settings Model */
		$settingsModel = $this->model('Settings');
        $data['settings'] = $settingsModel->get();
		
		/* Use Input Library */
		$input = $this->library('Input');
		
		
		//Edit Profile Data
		if(isset($_POST['post_theme'])){
		 if ($input->exists()) {
				
				
				$update = $settingsModel->theme($input->get('name'));
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['theme_selected_successfully']];
				    redirect('admin/dashboard');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made_theme']];
				    redirect('admin/dashboard');
				}
				
         }
		}			
		
        return ['content' => $this->view->render($data, 'admin/dashboard')];
    }

    /**
     * Get the available Themes
     */
    private function getThemes() {
        $path = sprintf('%s/../../%s/%s/', __DIR__, PUBLIC_PATH, THEME_PATH);

        $themes = scandir($path);

        $output = [];
        foreach($themes as $theme) {
            // Check if the theme has an info.php file a && file_exists($path.$theme.'/icon.png)nd a thumbnail
            if(file_exists($path.$theme.'/info.php') && file_exists($path.$theme.'/icon.png')) {
                // Store the theme information
                require($path.$theme.'/info.php');
                $output[$theme]['name']     = $name;
                $output[$theme]['author']   = $author;
                $output[$theme]['url']      = $url;
                $output[$theme]['version']  = $version;
                $output[$theme]['bootstrap']  = $bootstrap;
                $output[$theme]['path']     = $theme;
            }
        }

        return $output;
    }
	

    /**
     * Boxcard Theme
     */
    public function boxcard()
    {

        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* URL */
		$data['m'] = $this->url[2];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Settings Model */
		$settingsModel = $this->model('Settings');
        $data['settings'] = $settingsModel->get();
		
		/* Use Theme Model */
		$themeModel = $this->model('Theme');
        $data['theme'] = $themeModel->get($data['settings']['theme']);
		
		/* Use Input Library */
		$input = $this->library('Input');
		
		//Details
		if(isset($_POST['edit_details'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
				  'title' => [
					 'required' => true,
				  ],
				  'sub_title' => [
					 'required' => true,
				  ],
			]);
				 
			if (!$validation->fails()) {
				
				$update = $themeModel->update($input->get('title'), $input->get('sub_title'), $input->get('id'));
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/boxcard/details');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/boxcard/details');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
					redirect('admin/boxcard/details');
		   }

		 }
		}
		


		/*Edit Author Image*/
		if(isset($_POST['edit_bgimage'])){
		 if ($input->exists()) {
			
			$valid_formats = array("jpg", "jpeg", "png", "gif", "bmp");
		   
			$name = $_FILES['photoimg']['name'];
			$size = $_FILES['photoimg']['size'];

			if(!empty($name))
			{
			  
			  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
			  
			  // If there is no error during upload and the file is PNG
			  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
			   {
				 $fileName = $this->rando().'.'.$fileFormat;
				 // If the file can't be written on the disk (will return 0)
				 $path = sprintf('%s/../../%s/%s/admin/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
				 

				 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
				  {

					// Get the old image
					$oldFileName = $input->get('bg_image_one') ?? null;

					// Remove the old variant of the image
					if($oldFileName && $oldFileName != $fileName) {
						unlink($path.$oldFileName);
					}
				
					$update = $themeModel->changeImage($fileName, $input->get('id'));
					
					if ($update == 1) {
						$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/boxcard/bgimage');
					} else {
						$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/boxcard/bgimage');
					}		  
							
				  }else{
					$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
					redirect('admin/boxcard/bgimage');
				  }
			   }else{
					$_SESSION['message'][] = ['error', $this->lang['format_error']];
					redirect('admin/boxcard/bgimage');
			   }
			  }else{
					$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
					redirect('admin/boxcard/bgimage');
			  }	
			
		 }	
		}
		


		/*Edit Author Image*/
		if(isset($_POST['edit_bgimage_two'])){
		 if ($input->exists()) {
			
			$valid_formats = array("jpg", "jpeg", "png", "gif", "bmp");
		   
			$name = $_FILES['photoimg']['name'];
			$size = $_FILES['photoimg']['size'];

			if(!empty($name))
			{
			  
			  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
			  
			  // If there is no error during upload and the file is PNG
			  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
			   {
				 $fileName = $this->rando().'.'.$fileFormat;
				 // If the file can't be written on the disk (will return 0)
				 $path = sprintf('%s/../../%s/%s/admin/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
				 

				 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
				  {

					// Get the old image
					$oldFileName = $input->get('bg_image_two') ?? null;

					// Remove the old variant of the image
					if($oldFileName && $oldFileName != $fileName) {
						unlink($path.$oldFileName);
					}
				
					$update = $themeModel->changeImageTwo($fileName, $input->get('id'));
					
					if ($update == 1) {
						$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/boxcard/bgimage_two');
					} else {
						$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/boxcard/bgimage_two');
					}		  
							
				  }else{
					$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
					redirect('admin/boxcard/bgimage_two');
				  }
			   }else{
					$_SESSION['message'][] = ['error', $this->lang['format_error']];
					redirect('admin/boxcard/bgimage_two');
			   }
			  }else{
					$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
					redirect('admin/boxcard/bgimage_two');
			  }	
			
		 }	
		}


		/*Edit Hero Link One Details*/
		if(isset($_POST['edit_hero_one_link'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
				  'hero_link_one' => [
					 'required' => true,
				  ]
			]);
				 
			if (!$validation->fails()) {
					
				$update = $themeModel->hero_one_link($input->get('hero_link_one'), $input->get('id'));
				
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
				redirect('admin/boxcard/hero_one');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
				redirect('admin/boxcard/hero_one');
				}		 
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
					redirect('admin/boxcard/hero_one');
		   }
			
		 }	
		}


		/*Edit Hero One Details*/
		if(isset($_POST['edit_hero_one'])){
		 if ($input->exists()) {
			
				$valid_formats = array("jpg", "jpeg", "png", "gif", "bmp");
			   
				$name = $_FILES['photoimg']['name'];
				$size = $_FILES['photoimg']['size'];

				if(!empty($name))
				{
				  
				  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
				  
				  // If there is no error during upload and the file is PNG
				  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
				   {
					 $fileName = $this->rando().'.'.$fileFormat;
					 // If the file can't be written on the disk (will return 0)
					 $path = sprintf('%s/../../%s/%s/admin/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
					 

					 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
					  {

						// Get the old image
						$oldFileName = $input->get('hero_image_one') ?? null;

						// Remove the old variant of the image
						if($oldFileName && $oldFileName != $fileName) {
							unlink($path.$oldFileName);
						}
					
						$update = $themeModel->hero_one($input->get('hero_link_one'), $fileName, $input->get('id'));
						
						if ($update == 1) {
							$_SESSION['message'][] = ['success', $this->lang['details_updated']];
						redirect('admin/boxcard/hero_one');
						} else {
							$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
						redirect('admin/boxcard/hero_one');
						}		  
								
					  }else{
						$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
						redirect('admin/boxcard/hero_one');
					  }
				   }else{
						$_SESSION['message'][] = ['error', $this->lang['format_error']];
						redirect('admin/boxcard/hero_one');
				   }
				  }else{
						$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
						redirect('admin/boxcard/hero_one');
				  }	
					
			
		 }	
		}


		/*Edit Hero two Details*/
		if(isset($_POST['edit_hero_two_link'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
				  'hero_link_two' => [
					 'required' => true,
				  ]
			]);
				 
			if (!$validation->fails()) {
					
				$update = $themeModel->hero_two_link($input->get('hero_link_two'), $input->get('id'));
				
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
				redirect('admin/boxcard/hero_two');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
				redirect('admin/boxcard/hero_two');
				}		  
							
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
						redirect('admin/boxcard/hero_two');
		   }
			
		 }	
		}	


		/*Edit Hero One Details*/
		if(isset($_POST['edit_hero_two'])){
		 if ($input->exists()) {
			
				$valid_formats = array("jpg", "jpeg", "png", "gif", "bmp");
			   
				$name = $_FILES['photoimg']['name'];
				$size = $_FILES['photoimg']['size'];

				if(!empty($name))
				{
				  
				  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
				  
				  // If there is no error during upload and the file is PNG
				  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
				   {
					 $fileName = $this->rando().'.'.$fileFormat;
					 // If the file can't be written on the disk (will return 0)
					 $path = sprintf('%s/../../%s/%s/admin/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
					 

					 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
					  {

						// Get the old image
						$oldFileName = $input->get('hero_image_two') ?? null;

						// Remove the old variant of the image
						if($oldFileName && $oldFileName != $fileName) {
							unlink($path.$oldFileName);
						}
					
						$update = $themeModel->hero_two($input->get('hero_link_two'), $fileName, $input->get('id'));
						
						if ($update == 1) {
							$_SESSION['message'][] = ['success', $this->lang['details_updated']];
						redirect('admin/boxcard/hero_two');
						} else {
							$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
						redirect('admin/boxcard/hero_two');
						}		  
								
					  }else{
						$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
						redirect('admin/boxcard/hero_two');
					  }
				   }else{
						$_SESSION['message'][] = ['error', $this->lang['format_error']];
						redirect('admin/boxcard/hero_two');
				   }
				  }else{
						$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
						redirect('admin/boxcard/hero_two');
				  }	
					
			
		 }	
		}				
				


        return ['content' => $this->view->render($data, 'admin/boxcard')];
    }
	

    /**
     * Boxsale Theme
     */
    public function boxsale()
    {

        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* URL */
		$data['m'] = $this->url[2];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Settings Model */
		$settingsModel = $this->model('Settings');
        $data['settings'] = $settingsModel->get();
		
		/* Use Theme Model */
		$themeModel = $this->model('Theme');
        $data['theme'] = $themeModel->get($data['settings']['theme']);
		
		/* Use Input Library */
		$input = $this->library('Input');
		
		//Details
		if(isset($_POST['edit_details'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
				  'title' => [
					 'required' => true,
				  ],
			]);
				 
			if (!$validation->fails()) {
				
				$update = $themeModel->update($input->get('title'), $input->get('sub_title'), $input->get('id'));
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/boxsale/details');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/boxsale/details');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
					redirect('admin/boxsale/details');
		   }

		 }
		}
		


		/*Edit Author Image*/
		if(isset($_POST['edit_bgimage'])){
		 if ($input->exists()) {
			
			$valid_formats = array("jpg", "jpeg", "png", "gif", "bmp");
		   
			$name = $_FILES['photoimg']['name'];
			$size = $_FILES['photoimg']['size'];

			if(!empty($name))
			{
			  
			  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
			  
			  // If there is no error during upload and the file is PNG
			  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
			   {
				 $fileName = $this->rando().'.'.$fileFormat;
				 // If the file can't be written on the disk (will return 0)
				 $path = sprintf('%s/../../%s/%s/admin/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
				 

				 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
				  {

					// Get the old image
					$oldFileName = $input->get('bg_image_one') ?? null;

					// Remove the old variant of the image
					if($oldFileName && $oldFileName != $fileName) {
						unlink($path.$oldFileName);
					}
				
					$update = $themeModel->changeImage($fileName, $input->get('id'));
					
					if ($update == 1) {
						$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/boxsale/bgimage');
					} else {
						$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/boxsale/bgimage');
					}		  
							
				  }else{
					$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
					redirect('admin/boxsale/bgimage');
				  }
			   }else{
					$_SESSION['message'][] = ['error', $this->lang['format_error']];
					redirect('admin/boxsale/bgimage');
			   }
			  }else{
					$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
					redirect('admin/boxsale/bgimage');
			  }	
			
		 }	
		}
		


		/*Edit Author Image*/
		if(isset($_POST['edit_bgimage_two'])){
		 if ($input->exists()) {
			
			$valid_formats = array("jpg", "jpeg", "png", "gif", "bmp");
		   
			$name = $_FILES['photoimg']['name'];
			$size = $_FILES['photoimg']['size'];

			if(!empty($name))
			{
			  
			  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
			  
			  // If there is no error during upload and the file is PNG
			  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
			   {
				 $fileName = $this->rando().'.'.$fileFormat;
				 // If the file can't be written on the disk (will return 0)
				 $path = sprintf('%s/../../%s/%s/admin/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
				 

				 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
				  {

					// Get the old image
					$oldFileName = $input->get('bg_image_two') ?? null;

					// Remove the old variant of the image
					if($oldFileName && $oldFileName != $fileName) {
						unlink($path.$oldFileName);
					}
				
					$update = $themeModel->changeImageTwo($fileName, $input->get('id'));
					
					if ($update == 1) {
						$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/boxsale/bgimage_two');
					} else {
						$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/boxsale/bgimage_two');
					}		  
							
				  }else{
					$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
					redirect('admin/boxsale/bgimage_two');
				  }
			   }else{
					$_SESSION['message'][] = ['error', $this->lang['format_error']];
					redirect('admin/boxsale/bgimage_two');
			   }
			  }else{
					$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
					redirect('admin/boxsale/bgimage_two');
			  }	
			
		 }	
		}

		/*Edit Author Image*/
		if(isset($_POST['edit_login_image'])){
		 if ($input->exists()) {
			
			$valid_formats = array("jpg", "jpeg", "png", "gif", "bmp");
		   
			$name = $_FILES['photoimg']['name'];
			$size = $_FILES['photoimg']['size'];

			if(!empty($name))
			{
			  
			  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
			  
			  // If there is no error during upload and the file is PNG
			  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
			   {
				 $fileName = $this->rando().'.'.$fileFormat;
				 // If the file can't be written on the disk (will return 0)
				 $path = sprintf('%s/../../%s/%s/admin/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
				 

				 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
				  {

					// Get the old image
					$oldFileName = $input->get('login_image') ?? null;

					// Remove the old variant of the image
					if($oldFileName && $oldFileName != $fileName) {
						unlink($path.$oldFileName);
					}
				
					$update = $themeModel->changeLoginImage($fileName, $input->get('id'));
					
					if ($update == 1) {
						$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/boxsale/login_image');
					} else {
						$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/boxsale/login_image');
					}		  
							
				  }else{
					$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
					redirect('admin/boxsale/login_image');
				  }
			   }else{
					$_SESSION['message'][] = ['error', $this->lang['format_error']];
					redirect('admin/boxsale/login_image');
			   }
			  }else{
					$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
					redirect('admin/boxsale/login_image');
			  }	
			
		 }	
		}

        return ['content' => $this->view->render($data, 'admin/boxsale')];
    }
	
    public function logout() {
        $admin = $this->library('Admin');
		
		$admin->logout();
		
		redirect('admin/login');
    }
	

    /**
     * @param $language string
     */
    private function updateLanguage($language)
    {
        setcookie('lang', $language, time() + (10 * 365 * 24 * 60 * 60), COOKIE_PATH);
        redirect('admin/dashboard');
    }



    public function profile() {
        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* URL */
		$data['m'] = $this->url[2];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Input Library */
		$input = $this->library('Input');
		
		/* Use Admin Model */
		$adminModel = $this->model('Admin');
		
		
		//Edit Profile Data
		if(isset($_POST['profile'])){
		 if ($input->exists()) {
			
			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
			  'username' => [
				 'required' => true,
				 'maxlength' => 20,
				 'minlength' => 2
			  ],
			  'name' => [
				 'required' => true,
				 'maxlength' => 100,
				 'minlength' => 2
			  ],
			  'email' => [
				 'required' => true,
				 'maxlength' => 255,
				 'email' => true
			   ]
			]);
				 
			if (!$validation->fails()) {
				
				
				$update = $adminModel->profileDetails($input->get('name'), $input->get('username'), $input->get('email'), $data['admin']['adminid']);
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
				    redirect('admin/profile/details');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
				    redirect('admin/profile/details');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/profile/details');
		   }
         }
		}	

		/*Edit Image Data*/
		if(isset($_POST['picture'])){
		 if ($input->exists()) {
			
			$valid_formats = array("jpg", "png", "gif", "bmp");
		   
			$name = $_FILES['photoimg']['name'];
			$size = $_FILES['photoimg']['size'];

			if(!empty($name))
			{
			  
			  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
			  
              // If there is no error during upload and the file is PNG
			  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
			   {
				 $fileName = $this->rando().'.'.$fileFormat;
				 // If the file can't be written on the disk (will return 0)
                 $path = sprintf('%s/../../%s/%s/admin/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
				 

				 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
				  {

					// Get the old image
					$oldFileName = $data['admin']['imagelocation'] ?? null;

					// Remove the old variant of the image
					if($oldFileName && $oldFileName != $fileName) {
						unlink($path.$oldFileName);
					}
					$update = $adminModel->profileImage($fileName, $data['admin']['adminid']); 
					
					if ($update == 1) {
						$_SESSION['message'][] = ['success', $this->lang['details_updated']];
						redirect('admin/profile/image');
					} else {
						$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
						redirect('admin/profile/image');
					}		  
							
				  }else{
					$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
						redirect('admin/profile/image');	
				  }
			   }else{
					$_SESSION['message'][] = ['error', $this->lang['format_error']];
						redirect('admin/profile/image');			
			   }
			  }else{
					$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
						redirect('admin/profile/image');
			  }	
			
		 }	
		}

		/*Edit Password Data*/
		if(isset($_POST['password'])){
		 if ($input->exists()) {
		 
			
			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
			  'password_current' => [
				 'required' => true,
				 'maxlength' => 300
			  ],
			   'password_new' => [
				 'required' => true,
				 'minlength' => 6
			   ],
			   'password_new_again' => [
				 'required' => true,
				 'match' => 'password_new'
			   ]
			]);
				 
			if (!$validation->fails()) {

				if (password_verify($input->get('password_current'), $data['admin']['password'])) {
					
					/* Hash Password */
					$password = password_hash($input->get('password_new'), PASSWORD_DEFAULT);
					
					$update = $adminModel->password($password, $data['admin']['adminid']);
						
					if ($update == 1) {
						$_SESSION['message'][] = ['success', $this->lang['details_updated']];
						redirect('admin/profile/password');
					} else {
						$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
						redirect('admin/profile/password');
					}
					
				} else {
					
					$_SESSION['message'][] = ['error', $this->lang['current_password_does_not_match']];
						redirect('admin/profile/password');
				 
				}
			  
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/profile/password');
		     }	
		  
			
		 }
		}		
		
        return ['content' => $this->view->render($data, 'admin/profile')];
    }
	
	
	/**
	 * Settings Function
	 */
    public function settings() {
        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* URL */
		$data['m'] = $this->url[2];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Input Library */
		$input = $this->library('Input');
		
		/* Use Admin Model */
		$adminModel = $this->model('Admin');
		
		/* Use Settings Model */
		$settingsModel = $this->model('Settings');
        $data['settings'] = $settingsModel->get();

		//Edit Site Settings Data
		if(isset($_POST['postsite'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
			  'sitename' => [
				 'required' => true,
				 'minlength' => 2
			  ],
			  'title' => [
				 'required' => true,
				 'minlength' => 2
			  ],
			  'description' => [
				 'required' => true,
				 'minlength' => 3
			   ],
			  'keywords' => [
				 'required' => true,
				 'minlength' => 3
			   ],
			  'timezone' => [
				 'required' => true,
			   ]
			]);
				 
			if (!$validation->fails()) {
				
				$update = $settingsModel->siteDetails($input->get('sitename'), $input->get('title'), $input->get('description'), $input->get('keywords'), $input->get('timezone'));
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/settings/site');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/settings/site');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/settings/site');
		   }

		 }
		}		

		/*Edit Image Data*/
		if(isset($_POST['postlogo'])){
		 if ($input->exists()) {
			
			$valid_formats = array("jpg", "jpeg", "png", "gif", "bmp");
		   
			$name = $_FILES['photoimg']['name'];
			$size = $_FILES['photoimg']['size'];

			if(!empty($name))
			{
			  
			  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
			  
              // If there is no error during upload and the file is PNG
			  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
			   {
				 $fileName = $this->rando().'.'.$fileFormat;
				 // If the file can't be written on the disk (will return 0)
                 $path = sprintf('%s/../../%s/%s/admin/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
				 

				 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
				  {

					// Get the old image
					$oldFileName = $data['settings']['logo'] ?? null;

					// Remove the old variant of the image
					if($oldFileName && $oldFileName != $fileName) {
						unlink($path.$oldFileName);
					}
					$update = $settingsModel->siteLogo($fileName); 
					
					if ($update == 1) {
						$_SESSION['message'][] = ['success', $this->lang['details_updated']];
						redirect('admin/settings/logo');
					} else {
						$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
						redirect('admin/settings/logo');
					}		  
							
				  }else{
					$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
						redirect('admin/settings/logo');	
				  }
			   }else{
					$_SESSION['message'][] = ['error', $this->lang['format_error']];
						redirect('admin/settings/logo');			
			   }
			  }else{
					$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
						redirect('admin/settings/logo');
			  }	
			
		 }	
		}

		/*Edit Favicon*/
		if(isset($_POST['postfavicon'])){
		 if ($input->exists()) {
			
			$valid_formats = array("jpg", "jpeg", "png", "gif", "bmp");
		   
			$name = $_FILES['photoimg']['name'];
			$size = $_FILES['photoimg']['size'];

			if(!empty($name))
			{
			  
			  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
			  
              // If there is no error during upload and the file is PNG
			  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
			   {
				 $fileName = $this->rando().'.'.$fileFormat;
				 // If the file can't be written on the disk (will return 0)
                 $path = sprintf('%s/../../%s/%s/admin/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
				 

				 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
				  {

					// Get the old image
					$oldFileName = $data['settings']['favicon'] ?? null;

					// Remove the old variant of the image
					if($oldFileName && $oldFileName != $fileName) {
						unlink($path.$oldFileName);
					}
					$update = $settingsModel->siteFavicon($fileName); 
					
					if ($update == 1) {
						$_SESSION['message'][] = ['success', $this->lang['details_updated']];
						redirect('admin/settings/favicon');
					} else {
						$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
						redirect('admin/settings/favicon');
					}		  
							
				  }else{
					$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
						redirect('admin/settings/favicon');	
				  }
			   }else{
					$_SESSION['message'][] = ['error', $this->lang['format_error']];
						redirect('admin/settings/favicon');			
			   }
			  }else{
					$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
						redirect('admin/settings/favicon');
			  }	
			
		 }	
		}

		//Edit Analytics
		if(isset($_POST['postanalytics'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
			  'analytics' => [
				 'required' => true,
			  ],
			]);
				 
			if (!$validation->fails()) {
				
				$update = $settingsModel->siteAnalytics($input->get('analytics'));
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/settings/analytics');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/settings/analytics');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/settings/analytics');
		   }

		 }
		}

		//Edit Analytics
		if(isset($_POST['post_contact'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
			  'contact_email' => [
				 'required' => true,
				 'email' => true,
			  ],
			  'contact_phone' => [
				 'required' => true,
			  ],
			  'contact_location' => [
				 'required' => true,
			  ],
			]);
				 
			if (!$validation->fails()) {
				
				$update = $settingsModel->contact($input->get('contact_email'), $input->get('contact_phone'), $input->get('contact_location'));
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/settings/contact');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/settings/contact');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/settings/contact');
		   }

		 }
		}
				
		
		
        return ['content' => $this->view->render($data, 'admin/settings')];
    }
	
	
	/**
	 * Email Settings Function
	 */
    public function email_settings() {
        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Input Library */
		$input = $this->library('Input');
		
		/* Use Admin Model */
		$adminModel = $this->model('Admin');
		
		/* Use Settings Model */
		$settingsModel = $this->model('Settings');
        $data['settings'] = $settingsModel->get();

		//Edit Email
		if(isset($_POST['post_email'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
			  'smtp_host' => [
				 'required' => true,
			  ],
			  'smtp_username' => [
				 'required' => true,
			  ],
			  'smtp_password' => [
				 'required' => true,
			  ],
			  'smtp_encryption' => [
				 'required' => true,
			  ],
			  'smtp_port' => [
				 'required' => true,
			  ],
			]);
				 
			if (!$validation->fails()) {
				
				$update = $settingsModel->email($input->get('smtp_host'), $input->get('smtp_username'), $input->get('smtp_password'), $input->get('smtp_encryption'), $input->get('smtp_port'));
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/email_settings');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/email_settings');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/email_settings');
		   }

		 }
		}		
		
		
        return ['content' => $this->view->render($data, 'admin/email_settings')];
    }
	
	
	/**
	 * Amazon Settings Function
	 */
    public function amazon() {
        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Input Library */
		$input = $this->library('Input');
		
		/* Use Admin Model */
		$adminModel = $this->model('Admin');
		
		/* Use Settings Model */
		$settingsModel = $this->model('Settings');
        $data['settings'] = $settingsModel->get();

		//Edit Amazon s3
		if(isset($_POST['post_s3'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
			  'bucket_name' => [
				 'required' => true,
			  ],
			  's3_key' => [
				 'required' => true,
			  ],
			  's3_secret' => [
				 'required' => true,
			  ],
			  's3_region' => [
				 'required' => true,
			  ],
			]);
				 
			if (!$validation->fails()) {
				
				$update = $settingsModel->s3($input->get('bucket_name'), $input->get('s3_key'), $input->get('s3_secret'), $input->get('s3_region'));
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/amazon');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/amazon');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/amazon');
		   }

		 }
		}		
		
		
        return ['content' => $this->view->render($data, 'admin/amazon')];
    }
	
	
	/**
	 * Payment Settings Function
	 */
    public function payment_settings() {
        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* URL */
		$data['m'] = $this->url[2];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Input Library */
		$input = $this->library('Input');
		
		/* Use Admin Model */
		$adminModel = $this->model('Admin');
		
		/* Use Settings Model */
		$settingsModel = $this->model('Settings');
        $data['settings'] = $settingsModel->get();

		//Edit PayPal Details
		if(isset($_POST['postpaypal'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
			  'paypal_active' => [
				 'required' => true,
			  ],
			  'sandbox' => [
				 'required' => true
			  ],
			  'paypal_email' => [
				 'required' => true,
				 'email' => true,
			  ]
			]);
				 
			if (!$validation->fails()) {
				
				$update = $settingsModel->paypal($input->get('paypal_active'), $input->get('sandbox'), $input->get('paypal_email'));
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/payment_settings/paypal');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/payment_settings/paypal');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/payment_settings/paypal');
		   }

		 }
		}	

		//Edit Stripe Details
		if(isset($_POST['poststripe'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
			  'stripe_active' => [
				 'required' => true,
			  ],
			  'stripe_secret_key' => [
				 'required' => true
			  ],
			  'stripe_public_key' => [
				 'required' => true,
			  ]
			]);
				 
			if (!$validation->fails()) {
				
				$update = $settingsModel->stripe($input->get('stripe_active'), $input->get('stripe_secret_key'), $input->get('stripe_public_key'));
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/payment_settings/stripe');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/payment_settings/stripe');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/payment_settings/stripe');
		   }

		 }
		}

		//Edit Razorpay Details
		if(isset($_POST['postrazorpay'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
			  'razorpay_active' => [
				 'required' => true,
			  ],
			  'razorpay_key_id' => [
				 'required' => true
			  ]
			]);
				 
			if (!$validation->fails()) {
				
				$update = $settingsModel->razorpay($input->get('razorpay_active'), $input->get('razorpay_key_id'));
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/payment_settings/razorpay');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/payment_settings/razorpay');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/payment_settings/razorpay');
		   }

		 }
		}	

		//Edit Paystack Details
		if(isset($_POST['post_paystack'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
			  'paystack_active' => [
				 'required' => true,
			  ],
			  'paystack_key' => [
				 'required' => true
			  ]
			]);
				 
			if (!$validation->fails()) {
				
				$update = $settingsModel->paystack($input->get('paystack_active'), $input->get('paystack_key'));
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/payment_settings/paystack');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/payment_settings/paystack');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/payment_settings/paystack');
		   }

		 }
		}		
		
		
        return ['content' => $this->view->render($data, 'admin/payment_settings')];
    }
	
	
	/**
	 * Settings Function
	 */
    public function currency_settings() {
        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* URL */
		$data['m'] = $this->url[2];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Input Library */
		$input = $this->library('Input');
		
		/* Use Admin Model */
		$adminModel = $this->model('Admin');
        $data['currency'] = $adminModel->currencyDetails();
		
		/* Use Settings Model */
		$settingsModel = $this->model('Settings');
        $data['settings'] = $settingsModel->get();
		
		//Edit Currency Data
		if(isset($_POST['currency_submit'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
			  'currency_select' => [
				 'required' => true,
			  ],
			]);
				 
			if (!$validation->fails()) {
				
				$update = $adminModel->currencySelect($input->get('currency_select'));
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/currency_settings/default');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/currency_settings/default');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/currency_settings/default');
		   }

		 }
		}	
		
		//Add Currency Data
		if(isset($_POST['postcurrency'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
				  'currency_code' => [
					 'required' => true,
				  ],
				  'currency_name' => [
					 'required' => true,
				  ],
				  'currency_symbol' => [
					 'required' => true,
				  ],
			]);
				 
			if (!$validation->fails()) {
				
				$insert = $adminModel->currencyAdd($input->get('currency_code'), $input->get('currency_name'), $input->get('currency_symbol'));
					
				if ($insert == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_added']];
					redirect('admin/currency_settings/currency');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['error_when_saving']];
					redirect('admin/currency_settings/currency');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/currency_settings/currency');
		   }

		 }
		}

        // Edit Page
        if(isset($this->url[2]) && $this->url[2] == 'edit') {			
			
            $has = $adminModel->currencyHas($this->url[3]);

            // If the currency requested exists
            if($has === true) {
				
                $data["currency"] = $adminModel->currencyGet($this->url[3]);
			

		
				//Add Currency Data
				if(isset($_POST['editcurrency'])){
				 if ($input->exists()) {

					$validator = $this->library('Validator');
					
					$validation = $validator->check($_POST, [
						  'currency_code' => [
							 'required' => true,
						  ],
						  'currency_name' => [
							 'required' => true,
						  ],
						  'currency_symbol' => [
							 'required' => true,
						  ],
					]);
						 
					if (!$validation->fails()) {
						
						$update = $adminModel->currencyUpdate($input->get('currency_code'), $input->get('currency_name'), $input->get('currency_symbol'), $input->get('currency_id'));
							
						if ($update == 1) {
							$_SESSION['message'][] = ['success', $this->lang['details_updated']];
							redirect('admin/currency_settings/edit/'. $input->get('currency_id'));
						} else {
							$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
							redirect('admin/currency_settings/edit/'. $input->get('currency_id'));
						}
							
					 } else {

						 foreach ($validation->errors()->all() as $err) {
							$str = implode(" ",$err);
							 foreach ($err as $r) {
								$_SESSION['errors'][] = ['error', $r];
							 }	
						 }
						 
							redirect('admin/currency_settings/edit/'. $input->get('currency_id'));
				   }

				 }
				}
				
                return ['content' => $this->view->render($data, 'admin/editcurrency')];
				
            } else {
                redirect('admin/currency_settings/currency');
            }
        }	
        // Delete
        elseif(isset($this->url[2]) && $this->url[2] == 'delete') {			
			
            $has = $adminModel->currencyHas($this->url[3]);

            // If the currency requested exists
            if($has === true) {
				
                $data["currency"] = $adminModel->currencyGet($this->url[3]);
		
				//Delete Data
				if(isset($_POST['deletecurrency'])){
				 if ($input->exists()) {

					$validator = $this->library('Validator');
					
					$validation = $validator->check($_POST, [
						  'currency_id' => [
							 'required' => true,
						  ],
					]);
						 
					if (!$validation->fails()) {
						
						$delete = $adminModel->currencyDelete($input->get('currency_id'));
							
						if ($delete == 1) {
							$_SESSION['message'][] = ['success', $this->lang['deleted_successfully']];
							redirect('admin/currency_settings/currency');
						} else {
							$_SESSION['message'][] = ['warning', $this->lang['unable_to_delete']];
							redirect('admin/currency_settings/currency');
						}
							
					 } else {

						 foreach ($validation->errors()->all() as $err) {
							$str = implode(" ",$err);
							 foreach ($err as $r) {
								$_SESSION['errors'][] = ['error', $r];
							 }	
						 }
						 
							redirect('admin/currency_settings/currency');
				   }

				 }
				}
			
				
                //return ['content' => $this->view->render($data, 'admin/currency_settings')];
				
            } else {
                redirect('admin/currency_settings/currency');
            }
        }	
		
        return ['content' => $this->view->render($data, 'admin/currency_settings')];
    }
	
	//Delete Currency
	public function deleteCurrency(){
		
        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* URL */
		$data['m'] = $this->url[2];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		
		/* Use Input Library */
		$input = $this->library('Input');
		
		/* Use Admin Model */
		$adminModel = $this->model('Admin');
        $data['currency'] = $adminModel->currencyDetails();
		
		/* Use Settings Model */
		$settingsModel = $this->model('Settings');
        $data['settings'] = $settingsModel->get();

			

				
				$delete = $_POST['delete']; 
						
						$delete = $adminModel->currencyDelete($delete);
				
				 if ($delete == 1) {
						$response['status']  = $this->lang['success'];
						$response['message'] = $this->lang['deleted_successfully'];
				 } else {
						$response['status']  = $this->lang['error'];
						$response['message'] = $this->lang['unable_to_delete'];
				 }	
				echo json_encode($response);
				
	}	
	
	

    /**
     * Add Author 
     */
    public function addauthor()
    {

        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Input Library */
		$input = $this->library('Input');
		
		/* Use Settings Model */
		$authorModel = $this->model('Author');

		/*Add Author*/
		if(isset($_POST['post_author'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
				  'name' => [
					 'required' => true,
				  ],
			]);
				 
			if (!$validation->fails()) {
			
				$valid_formats = array("jpg", "jpeg", "png", "gif", "bmp");
			   
				$name = $_FILES['photoimg']['name'];
				$size = $_FILES['photoimg']['size'];

				if(!empty($name))
				{
				  
				  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
				  
				  // If there is no error during upload and the file is PNG
				  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
				   {
					 $fileName = $this->rando().'.'.$fileFormat;
					 // If the file can't be written on the disk (will return 0)
					 $path = sprintf('%s/../../%s/%s/admin/authors/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
					 

					 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
					  {

						$insert = $authorModel->addauthor($this->uniqueid(), $input->get('name'), $fileName); 
						
						if ($insert == 1) {
							$_SESSION['message'][] = ['success', $this->lang['details_added']];
							redirect('admin/addauthor');
						} else {
							$_SESSION['message'][] = ['warning', $this->lang['error_when_saving']];
							redirect('admin/addauthor');
						}		  
								
					  }else{
						$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
							redirect('admin/addauthor');	
					  }
				   }else{
						$_SESSION['message'][] = ['error', $this->lang['format_error']];
							redirect('admin/addauthor');			
				   }
				  }else{
						$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
							redirect('admin/addauthor');
				  }	
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/addauthor');
		   }
			
		 }	
		}	


        return ['content' => $this->view->render($data, 'admin/addauthor')];
    }	
	

    /**
     * Author List
     */
    public function authorlist()
    {

        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Settings Model */
		$authorModel = $this->model('Author');
        $data['author'] = $authorModel->authorDetails();
		
		/* Use Input Library */
		$input = $this->library('Input');

        // Edit Author
        if(isset($this->url[2]) && $this->url[2] == 'edit') {			
			
            $has = $authorModel->authorHas($this->url[4]);

            // If the currency requested exists
            if($has === true) {
				
				/* URL */
				$data['m'] = $this->url[3];
				
				/* Author data */
                $data["author"] = $authorModel->authorGet($this->url[4]);
			

		
				//Add Currency Data
				if(isset($_POST['editauthor'])){
				 if ($input->exists()) {

					$validator = $this->library('Validator');
					
					$validation = $validator->check($_POST, [
						  'name' => [
							 'required' => true,
						  ],
					]);
						 
					if (!$validation->fails()) {
						
						$update = $authorModel->authorUpdate($input->get('name'), $input->get('id'));
							
						if ($update == 1) {
							$_SESSION['message'][] = ['success', $this->lang['details_updated']];
							redirect('admin/authorlist/edit/profile/'. $input->get('id'));
						} else {
							$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
							redirect('admin/authorlist/edit/profile/'. $input->get('id'));
						}
							
					 } else {

						 foreach ($validation->errors()->all() as $err) {
							$str = implode(" ",$err);
							 foreach ($err as $r) {
								$_SESSION['errors'][] = ['error', $r];
							 }	
						 }
						 
							redirect('admin/authorlist/edit/profile/'. $input->get('id'));
				   }

				 }
				}
				


				/*Edit Author Image*/
				if(isset($_POST['author_image'])){
				 if ($input->exists()) {
					
					$valid_formats = array("jpg", "jpeg", "png", "gif", "bmp");
				   
					$name = $_FILES['photoimg']['name'];
					$size = $_FILES['photoimg']['size'];

					if(!empty($name))
					{
					  
					  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
					  
					  // If there is no error during upload and the file is PNG
					  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
					   {
						 $fileName = $this->rando().'.'.$fileFormat;
						 // If the file can't be written on the disk (will return 0)
						 $path = sprintf('%s/../../%s/%s/admin/authors/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
						 

						 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
						  {

							// Get the old image
							$oldFileName = $input->get('imagelocation') ?? null;

							// Remove the old variant of the image
							if($oldFileName && $oldFileName != $fileName) {
								unlink($path.$oldFileName);
							}
						
							$update = $authorModel->changeImage($fileName, $input->get('id'));
							
							if ($update == 1) {
								$_SESSION['message'][] = ['success', $this->lang['details_updated']];
							    redirect('admin/authorlist/edit/image/'. $input->get('id'));
							} else {
								$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
							    redirect('admin/authorlist/edit/image/'. $input->get('id'));
							}		  
									
						  }else{
							$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
							redirect('admin/authorlist/edit/image/'. $input->get('id'));
						  }
					   }else{
							$_SESSION['message'][] = ['error', $this->lang['format_error']];
							redirect('admin/authorlist/edit/image/'. $input->get('id'));
					   }
					  }else{
							$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
							redirect('admin/authorlist/edit/image/'. $input->get('id'));
					  }	
					
				 }	
				}				
				
                return ['content' => $this->view->render($data, 'admin/editauthor')];
				
            } else {
                redirect('admin/authorlist');
            }
        }	


        return ['content' => $this->view->render($data, 'admin/authorlist')];
    }
	
	

    /**
     * Add Technology
     */
    public function addtechnology()
    {

        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Input Library */
		$input = $this->library('Input');
		
		/* Use Settings Model */
		$technologyModel = $this->model('Technology');

		/*Add Author*/
		if(isset($_POST['add_technology'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
				  'name' => [
					 'required' => true,
				  ],
			]);
				 
			if (!$validation->fails()) {
			
				$valid_formats = array("jpg", "jpeg", "png", "gif", "bmp");
			   
				$name = $_FILES['photoimg']['name'];
				$size = $_FILES['photoimg']['size'];

				if(!empty($name))
				{
				  
				  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
				  
				  // If there is no error during upload and the file is PNG
				  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
				   {
					 $fileName = $this->rando().'.'.$fileFormat;
					 // If the file can't be written on the disk (will return 0)
					 $path = sprintf('%s/../../%s/%s/admin/technologies/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
					 

					 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
					  {

						$insert = $technologyModel->addtechnology($input->get('name'), $fileName); 
						
						if ($insert == 1) {
							$_SESSION['message'][] = ['success', $this->lang['details_added']];
							redirect('admin/addtechnology');
						} else {
							$_SESSION['message'][] = ['warning', $this->lang['error_when_saving']];
							redirect('admin/addtechnology');
						}		  
								
					  }else{
						$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
							redirect('admin/addtechnology');	
					  }
				   }else{
						$_SESSION['message'][] = ['error', $this->lang['format_error']];
							redirect('admin/addtechnology');			
				   }
				  }else{
						$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
							redirect('admin/addtechnology');
				  }	
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/addtechnology');
		   }
			
		 }	
		}	


        return ['content' => $this->view->render($data, 'admin/addtechnology')];
    }	
	

    /**
     * Technology List
     */
    public function technologies()
    {

        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Technology Model */
		$technologyModel = $this->model('Technology');
        $data['technology'] = $technologyModel->technologyDetails();
		
		/* Use Input Library */
		$input = $this->library('Input');

        // Edit Author
        if(isset($this->url[2]) && $this->url[2] == 'edit') {			
			
            $has = $technologyModel->technologyHas($this->url[4]);

            // If the currency requested exists
            if($has === true) {
				
				/* URL */
				$data['m'] = $this->url[3];
				
				/* Author data */
                $data["technology"] = $technologyModel->technologyGet($this->url[4]);
			

		
				//Add Currency Data
				if(isset($_POST['edit_details'])){
				 if ($input->exists()) {

					$validator = $this->library('Validator');
					
					$validation = $validator->check($_POST, [
						  'name' => [
							 'required' => true,
						  ],
					]);
						 
					if (!$validation->fails()) {
						
						$update = $technologyModel->technologyUpdate($input->get('name'), $input->get('id'));
							
						if ($update == 1) {
							$_SESSION['message'][] = ['success', $this->lang['details_updated']];
							redirect('admin/technologies/edit/details/'. $input->get('id'));
						} else {
							$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
							redirect('admin/technologies/edit/details/'. $input->get('id'));
						}
							
					 } else {

						 foreach ($validation->errors()->all() as $err) {
							$str = implode(" ",$err);
							 foreach ($err as $r) {
								$_SESSION['errors'][] = ['error', $r];
							 }	
						 }
						 
							redirect('admin/technologies/edit/details/'. $input->get('id'));
				   }

				 }
				}
				


				/*Edit Technology Image*/
				if(isset($_POST['edit_image'])){
				 if ($input->exists()) {
					
					$valid_formats = array("jpg", "jpeg", "png", "gif", "bmp");
				   
					$name = $_FILES['photoimg']['name'];
					$size = $_FILES['photoimg']['size'];

					if(!empty($name))
					{
					  
					  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
					  
					  // If there is no error during upload and the file is PNG
					  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
					   {
						 $fileName = $this->rando().'.'.$fileFormat;
						 // If the file can't be written on the disk (will return 0)
						 $path = sprintf('%s/../../%s/%s/admin/technologies/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
						 

						 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
						  {

							// Get the old image
							$oldFileName = $input->get('imagelocation') ?? null;

							// Remove the old variant of the image
							if($oldFileName && $oldFileName != $fileName) {
								unlink($path.$oldFileName);
							}
						
							$update = $technologyModel->changeImage($fileName, $input->get('id'));
							
							if ($update == 1) {
								$_SESSION['message'][] = ['success', $this->lang['details_updated']];
							    redirect('admin/technologies/edit/image/'. $input->get('id'));
							} else {
								$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
							    redirect('admin/technologies/edit/image/'. $input->get('id'));
							}		  
									
						  }else{
							$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
							    redirect('admin/technologies/edit/image/'. $input->get('id'));
						  }
					   }else{
							$_SESSION['message'][] = ['error', $this->lang['format_error']];
							    redirect('admin/technologies/edit/image/'. $input->get('id'));
					   }
					  }else{
							$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
							    redirect('admin/technologies/edit/image/'. $input->get('id'));
					  }	
					
				 }	
				}				
				
                return ['content' => $this->view->render($data, 'admin/edit_technology')];
				
            } else {
                redirect('admin/technologies');
            }
        }	


        return ['content' => $this->view->render($data, 'admin/technologies')];
    }

	

    /**
     * Add User
     */
    public function adduser()
    {

        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Input Library */
		$input = $this->library('Input');
		
		/* Use User Model */
		$userModel = $this->model('User');

		/*Add Author*/
		if(isset($_POST['add_user'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
				  'name' => [
					 'required' => true,
				  ],
				  'email' => [
					 'required' => true,
					 'email' => true,
				  ],
				  'password' => [
					 'required' => true,
				  ],
			]);
				 
			if (!$validation->fails()) {
		
					/* Hass Password */
					$password = password_hash($input->get('password'), PASSWORD_DEFAULT);
					
					/* Unique ID */	
                    $userid = $this->uniqueid();					
			
					$Insert = $userModel->add($userid, $input->get('name'), $input->get('email'), $password); 
					
					if ($Insert == 1) {
						$_SESSION['message'][] = ['success', $this->lang['details_saved']];
						redirect('admin/adduser');
					} else {
						$_SESSION['message'][] = ['warning', $this->lang['error_when_saving']];
						redirect('admin/adduser');
					}		
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/adduser');
		   }
			
		 }	
		}	


        return ['content' => $this->view->render($data, 'admin/adduser')];
    }		
	
	

    /**
     * User List
     */
    public function users()
    {

        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use User Model */
		$userModel = $this->model('User');
        $data['user'] = $userModel->details();
		
		/* Use Input Library */
		$input = $this->library('Input');

        // Edit
        if(isset($this->url[2]) && $this->url[2] == 'edit') {			
			
            $has = $userModel->has($this->url[4]);

            // If the currency requested exists
            if($has === true) {
				
				/* URL */
				$data['m'] = $this->url[3];
				
				/* User data */
                $data["user"] = $userModel->get($this->url[4]);
			

		
				//Edit
				if(isset($_POST['edit_details'])){
				 if ($input->exists()) {

					$validator = $this->library('Validator');
					
					$validation = $validator->check($_POST, [
						  'name' => [
							 'required' => true,
						  ],
						  'email' => [
							 'required' => true,
							 'email' => true,
						  ],
					]);
						 
					if (!$validation->fails()) {
						
						$update = $userModel->update($input->get('name'), $input->get('email'), $input->get('userid'));
							
						if ($update == 1) {
							$_SESSION['message'][] = ['success', $this->lang['details_updated']];
							redirect('admin/users/edit/details/'. $input->get('userid'));
						} else {
							$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
							redirect('admin/users/edit/details/'. $input->get('userid'));
						}
							
					 } else {

						 foreach ($validation->errors()->all() as $err) {
							$str = implode(" ",$err);
							 foreach ($err as $r) {
								$_SESSION['errors'][] = ['error', $r];
							 }	
						 }
						 
							redirect('admin/users/edit/details/'. $input->get('userid'));
				   }

				 }
				}
				


				/*Edit User Image*/
				if(isset($_POST['edit_image'])){
				 if ($input->exists()) {
					
					$valid_formats = array("jpg", "jpeg", "png", "gif", "bmp");
				   
					$name = $_FILES['photoimg']['name'];
					$size = $_FILES['photoimg']['size'];

					if(!empty($name))
					{
					  
					  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
					  
					  // If there is no error during upload and the file is PNG
					  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
					   {
						 $fileName = $this->rando().'.'.$fileFormat;
						 // If the file can't be written on the disk (will return 0)
						 $path = sprintf('%s/../../%s/%s/admin/users/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
						 

						 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
						  {

							// Get the old image
							$oldFileName = $input->get('imagelocation') ?? null;

							// Remove the old variant of the image
							if($input->get('imagelocation') != "default.png"):
								if($oldFileName && $oldFileName != $fileName) {
									unlink($path.$oldFileName);
								}
							endif;	
						
							$update = $userModel->changeImage($fileName, $input->get('userid'));
							
							if ($update == 1) {
								$_SESSION['message'][] = ['success', $this->lang['details_updated']];
							    redirect('admin/users/edit/image/'. $input->get('userid'));
							} else {
								$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
							    redirect('admin/users/edit/image/'. $input->get('userid'));
							}		  
									
						  }else{
							$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
							    redirect('admin/users/edit/image/'. $input->get('userid'));
						  }
					   }else{
							$_SESSION['message'][] = ['error', $this->lang['format_error']];
							    redirect('admin/users/edit/image/'. $input->get('userid'));
					   }
					  }else{
							$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
							    redirect('admin/users/edit/image/'. $input->get('userid'));
					  }	
					
				 }	
				}	
		
				//Edit
				if(isset($_POST['edit_password'])){
				 if ($input->exists()) {

					$validator = $this->library('Validator');
					
					$validation = $validator->check($_POST, [
						  'password_current' => [
							 'required' => true,
						  ],
						   'password' => [
							 'required' => true,
						   ],
						   'confirmPassword' => [
							 'required' => true,
							 'match' => 'password'
						   ]
					]);
						 
					if (!$validation->fails()) {
  	
						if (password_verify($input->get('password_current'), $input->get('db_password'))) {
							
							/* Hash Password */
							$password = password_hash($input->get('password'), PASSWORD_DEFAULT);
							
							$update = $userModel->password($password, $input->get('userid'));
								
							if ($update == 1) {
								$_SESSION['message'][] = ['success', $this->lang['details_updated']];
								redirect('admin/users/edit/password/'. $input->get('userid'));
							} else {
								$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
								redirect('admin/users/edit/password/'. $input->get('userid'));
							}
							
						} else {
							
							$_SESSION['message'][] = ['error', $this->lang['current_password_does_not_match']];
							redirect('admin/users/edit/password/'. $input->get('userid'));
						 
						}
							
					 } else {

						 foreach ($validation->errors()->all() as $err) {
							$str = implode(" ",$err);
							 foreach ($err as $r) {
								$_SESSION['errors'][] = ['error', $r];
							 }	
						 }
						 
							redirect('admin/users/edit/details/'. $input->get('userid'));
				   }

				 }
				}			
				
                return ['content' => $this->view->render($data, 'admin/edit_user')];
				
            } else {
                redirect('admin/users');
            }
        }	


        return ['content' => $this->view->render($data, 'admin/users')];
    }


    /**
     * S3 Files
     */
    public function s3()
    {

        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Settings Model */
		$authorModel = $this->model('Author');
        $data['author'] = $authorModel->authorDetails();
		
		/* Use Settings Model */
		$settingsModel = $this->model('Settings');
        $data['settings'] = $settingsModel->get();
		
		/* Use Input Library */
		$input = $this->library('Input');	
		
        // Edit
        if(isset($this->url[2]) && $this->url[2] == 'upload') {				
		
				
                return ['content' => $this->view->render($data, 'admin/upload')];
				
        }	
		
        // Edit
        if(isset($this->url[2]) && $this->url[2] == 'test') {	
		
					//CHANGE YOUR SETTINGS HERE//
					$buckutName = $data['settings']['bucket_name']; // S3 Bucket Name
					//Help: https://docs.aws.amazon.com/AmazonS3/latest/user-guide/view-bucket-properties.html
					$s3Reagon = $data['settings']['s3_region']; //S3 Region
					//Help: https://docs.aws.amazon.com/general/latest/gr/rande.html#s3_region
					$sbKey = $data['settings']['s3_key']; //S3 Key
					$sbSecret = $data['settings']['s3_secret']; //S3 Secret

					$s3 = new S3Client(['region' => $s3Reagon, 'version' => 'latest', 'credentials' => ['key' => $sbKey, 'secret' => $sbSecret, ]]);
					
				    $CurrDir = $_POST['dir'];
				    $uploadFileName = $_FILES['file']['name'];
				    $temp_file_location = $_FILES['file']['tmp_name'];
				    $result = $s3->putObject(['Bucket' => $buckutName, 'Key' => $CurrDir . $uploadFileName, 'SourceFile' => $temp_file_location, 'ACL' => 'public-read']);
					
				
        }
		
        // Delete
        if(isset($this->url[2]) && $this->url[2] == 'delete') {	
		
					//CHANGE YOUR SETTINGS HERE//
					$buckutName = $data['settings']['bucket_name']; // S3 Bucket Name
					//Help: https://docs.aws.amazon.com/AmazonS3/latest/user-guide/view-bucket-properties.html
					$s3Reagon = $data['settings']['s3_region']; //S3 Region
					//Help: https://docs.aws.amazon.com/general/latest/gr/rande.html#s3_region
					$sbKey = $data['settings']['s3_key']; //S3 Key
					$sbSecret = $data['settings']['s3_secret']; //S3 Secret

					$s3 = new S3Client(['region' => $s3Reagon, 'version' => 'latest', 'credentials' => ['key' => $sbKey, 'secret' => $sbSecret, ]]);
					
				    $fileName = strip_tags(stripslashes($_GET['Key']));
					$file = $s3->deleteObject(['Bucket' => $buckutName, 'Key' => $fileName]);
					
						 
							redirect('admin/s3/files');
        }	
		
        // Delete
        if(isset($this->url[2]) && $this->url[2] == 'download') {	
		
					//CHANGE YOUR SETTINGS HERE//
					$buckutName = $data['settings']['bucket_name']; // S3 Bucket Name
					//Help: https://docs.aws.amazon.com/AmazonS3/latest/user-guide/view-bucket-properties.html
					$s3Reagon = $data['settings']['s3_region']; //S3 Region
					//Help: https://docs.aws.amazon.com/general/latest/gr/rande.html#s3_region
					$sbKey = $data['settings']['s3_key']; //S3 Key
					$sbSecret = $data['settings']['s3_secret']; //S3 Secret

					$fileName = strip_tags(stripslashes($_GET['Key']));
					$filepath = "https://" . $buckutName . ".s3." . $s3Reagon . ".amazonaws.com/" . $fileName;					
					
				    
					// Process download
					header('Content-Type: application/octet-stream');
					header("Content-Transfer-Encoding: Binary"); 
					header("Content-disposition: attachment; filename=\"" . basename($filepath) . "\""); 
					readfile($filepath);  
					exit;
					
						 
							redirect('admin/s3/files');
        }		

        return ['content' => $this->view->render($data, 'admin/s3')];
    }	


    /**
     *File Info
     */
    public function fileinfo()
    {

        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];	
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Settings Model */
		$settingsModel = $this->model('Settings');
        $data['settings'] = $settingsModel->get();

        return ['content' => $this->view->render($data, 'admin/fileinfo')];
    }
	
	
    /**
     *Add Product
     */
    public function addproduct()
    {

        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Author Model */
		$authorModel = $this->model('Author');
        $data['author'] = $authorModel->authorDetails();
		
		/* Use Technology Model */
		$technologyModel = $this->model('Technology');
        $data['technology'] = $technologyModel->technologyDetails();
		
		/* Use Product Model */
		$productModel = $this->model('Product');
		
		/* Use Input Library */
		$input = $this->library('Input');	

		/*Add Product*/
		if(isset($_POST['post_product'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
					  'name' => [
						 'required' => true,
						 'minlength' => 2,
					   ],
					  'price' => [
						 'required' => true,
						 'minlength' => 2,
					   ],
					  'old_price' => [
						 'required' => true,
						 'minlength' => 2,
					   ],
					  'version' => [
						 'required' => true,
					   ],
					  'live_preview' => [
						 'required' => true,
						 'minlength' => 2,
					   ],
					  's3_link' => [
						 'required' => true,
						 'minlength' => 2,
					   ],
					  'authorid' => [
						 'required' => true,
						 'minlength' => 2,
					   ],
					  'technologies[]' => [
						 'required' => true,
					   ],
					  'description' => [
						 'required' => true,
						 'minlength' => 2,
					   ],
			]);
				 
			if (!$validation->fails()) {
			
				$valid_formats = array("jpg", "jpeg", "png", "gif", "bmp");
			   
				$name = $_FILES['photoimg']['name'];
				$size = $_FILES['photoimg']['size'];

				if(!empty($name))
				{
				  
				  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
				  
				  // If there is no error during upload and the file is PNG
				  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
				   {
					 $fileName = $this->rando().'.'.$fileFormat;
					 // If the file can't be written on the disk (will return 0)
					 $path = sprintf('%s/../../%s/%s/admin/products/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
					 

					 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
					  {
						  
					   $skills = $input->get('technologies');
					   $choice2 =implode(',',$skills);
					   
						$slug = $this->slugify($input->get('name'));
                        $productid = $this->uniqueid();						

						$insert = $productModel->add(
						                             $productid, 
						                             $input->get('name'), 
													 $slug, 
						                             $input->get('price'), 
						                             $input->get('old_price'), 
													 $input->get('version'), 
													 $input->get('live_preview'), 
													 $input->get('s3_link'), 
													 $input->get('authorid'), 
													 $choice2,
													 $fileName,
													 $input->get('featured'),
													 $input->get('description')
													 ); 
						
						if ($insert == 1) {
							$_SESSION['message'][] = ['success', $this->lang['details_updated']];
							redirect('admin/addproduct');
						} else {
							$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
							redirect('admin/addproduct');
						}		  
								
					  }else{
						$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
							redirect('admin/addproduct');	
					  }
				   }else{
						$_SESSION['message'][] = ['error', $this->lang['format_error']];
							redirect('admin/addproduct');			
				   }
				  }else{
						$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
							redirect('admin/addproduct');
				  }	
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
				 redirect('admin/addproduct');
		   }
			
		 }	
		}

        return ['content' => $this->view->render($data, 'admin/addproduct')];
    }	
	
    /**
     * Products
     */
    public function products()
    {

        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Product Model */
		$productModel = $this->model('Product');
        $data['product'] = $productModel->Details();
		
		/* Use Author Model */
		$authorModel = $this->model('Author');
        $data['author'] = $authorModel->authorDetails();
		
		/* Use Settings Model */
		$settingsModel = $this->model('Settings');
        $data['currency_symbol'] = $settingsModel->getCurrency();
		
		/* Use Technology Model */
		$technologyModel = $this->model('Technology');
        $data['technology_array'] = $technologyModel->getarray();
		
		/* Use Input Library */
		$input = $this->library('Input');

        // Edit
        if(isset($this->url[2]) && $this->url[2] == 'edit') {			
			
            $has = $productModel->Has($this->url[4]);

            // If the currency requested exists
            if($has === true) {
				
				/* URL */
				$data['m'] = $this->url[3];
				
				/* Product data */
                $data["product"] = $productModel->get($this->url[4]);
			

		
				//Edit
				if(isset($_POST['edit_details'])){
				 if ($input->exists()) {

					$validator = $this->library('Validator');
					
					$validation = $validator->check($_POST, [
					  'name' => [
						 'required' => true,
						 'minlength' => 2,
					   ],
					  'version' => [
						 'required' => true,
					   ],
					  'live_preview' => [
						 'required' => true,
						 'minlength' => 2,
					   ],
					  's3_link' => [
						 'required' => true,
						 'minlength' => 2,
					   ],
					  'authorid' => [
						 'required' => true,
						 'minlength' => 2,
					   ],
					  'technologies[]' => [
						 'required' => true,
					   ],
					]);
						 
					if (!$validation->fails()) {
						  
					    $skills = $input->get('technologies');
					    $choice2 =implode(',',$skills);
					   
						$slug = $this->slugify($input->get('name'));				

						$update = $productModel->update(
						                             $input->get('name'), 
													 $slug, 
						                             $input->get('price'), 
						                             $input->get('old_price'), 
													 $input->get('version'), 
													 $input->get('live_preview'), 
													 $input->get('s3_link'), 
													 $input->get('authorid'), 
													 $choice2,
													 $input->get('featured'),
													 $input->get('productid')
													 ); 
						
							
						if ($update == 1) {
							$_SESSION['message'][] = ['success', $this->lang['details_updated']];
							redirect('admin/products/edit/details/'. $input->get('productid'));
						} else {
							$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
							redirect('admin/products/edit/details/'. $input->get('productid'));
						}
							
					 } else {

						 foreach ($validation->errors()->all() as $err) {
							$str = implode(" ",$err);
							 foreach ($err as $r) {
								$_SESSION['errors'][] = ['error', $r];
							 }	
						 }
						 
							redirect('admin/products/edit/details/'. $input->get('productid'));
				   }

				 }
				}
		
				//Edit
				if(isset($_POST['edit_description'])){
				 if ($input->exists()) {

					$validator = $this->library('Validator');
					
					$validation = $validator->check($_POST, [
					  'description' => [
						 'required' => true,
					   ],
					]);
						 
					if (!$validation->fails()) {			

						$update = $productModel->updateDescription($input->get('description'), $input->get('productid')); 
						
							
						if ($update == 1) {
							$_SESSION['message'][] = ['success', $this->lang['details_updated']];
							redirect('admin/products/edit/description/'. $input->get('productid'));
						} else {
							$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
							redirect('admin/products/edit/description/'. $input->get('productid'));
						}
							
					 } else {

						 foreach ($validation->errors()->all() as $err) {
							$str = implode(" ",$err);
							 foreach ($err as $r) {
								$_SESSION['errors'][] = ['error', $r];
							 }	
						 }
						 
							redirect('admin/products/edit/description/'. $input->get('productid'));
				   }

				 }
				}
				


				/*Edit Technology Image*/
				if(isset($_POST['edit_image'])){
				 if ($input->exists()) {
					
					$valid_formats = array("jpg", "jpeg", "png", "gif", "bmp");
				   
					$name = $_FILES['photoimg']['name'];
					$size = $_FILES['photoimg']['size'];

					if(!empty($name))
					{
					  
					  $fileFormat = pathinfo($_FILES['photoimg']['name'], PATHINFO_EXTENSION);
					  
					  // If there is no error during upload and the file is PNG
					  if($_FILES['photoimg']['error'] == 0 && in_array($fileFormat, $valid_formats))
					   {
						 $fileName = $this->rando().'.'.$fileFormat;
						 // If the file can't be written on the disk (will return 0)
						 $path = sprintf('%s/../../%s/%s/admin/products/', __DIR__, PUBLIC_PATH, UPLOADS_PATH);
						 

						 if(move_uploaded_file($_FILES['photoimg']['tmp_name'], $path.$fileName))
						  {

							// Get the old image
							$oldFileName = $input->get('imagelocation') ?? null;

							// Remove the old variant of the image
							if($oldFileName && $oldFileName != $fileName) {
								unlink($path.$oldFileName);
							}
						
							$update = $productModel->changeImage($fileName, $input->get('productid'));
							
							if ($update == 1) {
								$_SESSION['message'][] = ['success', $this->lang['details_updated']];
							    redirect('admin/products/edit/image/'. $input->get('productid'));
							} else {
								$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
							    redirect('admin/products/edit/image/'. $input->get('productid'));
							}		  
									
						  }else{
							$_SESSION['message'][] = ['error', $this->lang['unable_to_upload_image']];
							    redirect('admin/products/edit/image/'. $input->get('productid'));
						  }
					   }else{
							$_SESSION['message'][] = ['error', $this->lang['format_error']];
							    redirect('admin/products/edit/image/'. $input->get('productid'));
					   }
					  }else{
							$_SESSION['message'][] = ['error', $this->lang['image_not_selected']];
							    redirect('admin/products/edit/image/'. $input->get('productid'));
					  }	
					
				 }	
				}				
				
                return ['content' => $this->view->render($data, 'admin/edit_product')];
				
            } else {
                redirect('admin/products');
            }
        }	


        return ['content' => $this->view->render($data, 'admin/products')];
    }	
	
    /**
     * Transactions
     */
    public function transactions()
    {

        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Transactions Model */
		$transactionsModel = $this->model('Transactions');
        $data['transactions'] = $transactionsModel->details();
		
		/* Use Product Model */
		$productModel = $this->model('Product');
        $data['product'] = $productModel->details();
		
		/* Use User Model */
		$userModel = $this->model('User');
        $data['user'] = $userModel->details();
		
		/* Use Settings Model */
		$settingsModel = $this->model('Settings');
        $data['currency_symbol'] = $settingsModel->getCurrency();
		

        return ['content' => $this->view->render($data, 'admin/transactions')];
    }	
	
    /**
     * Searches
     */
    public function searches()
    {

        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* Use Product Model */
		$productModel = $this->model('Product');
        $data['searches'] = $productModel->searches();
		

        return ['content' => $this->view->render($data, 'admin/searches')];
    }

	/**
     * Pages
     */
    public function pages()
    {

        /**
         * The $data array stores all the data that is passed to the views
         */
        $data = [];
		
		/* Use Admin Library */
        $admin = $this->library('Admin');
		$data['admin'] = $admin->data();
		if(!$admin->isLoggedIn()):
		 redirect('admin/login');
		endif;
		
		/* URL */
		$data['m'] = $this->url[2];
		
		/* Use Settings Model */
		$settingsModel = $this->model('Settings');
        $data['settings'] = $settingsModel->get();
		
		/* Use Input Library */
		$input = $this->library('Input');
		
		//Refund
		if(isset($_POST['post_refund'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
				  'description' => [
					 'required' => true,
				  ],
			]);
				 
			if (!$validation->fails()) {
				
				$update = $settingsModel->updateRefund($input->get('description'));
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/pages/refund');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/pages/refund');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
					redirect('admin/pages/refund');
		   }

		 }
		}
		
		//Terms
		if(isset($_POST['post_terms'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
				  'description' => [
					 'required' => true,
				  ],
			]);
				 
			if (!$validation->fails()) {
				
				$update = $settingsModel->updateTerms($input->get('description'));
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/pages/terms');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/pages/terms');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
					redirect('admin/pages/terms');
		   }

		 }
		}
		
		//Privacy
		if(isset($_POST['post_privacy'])){
		 if ($input->exists()) {

			$validator = $this->library('Validator');
			
			$validation = $validator->check($_POST, [
				  'description' => [
					 'required' => true,
				  ],
			]);
				 
			if (!$validation->fails()) {
				
				$update = $settingsModel->updatePrivacy($input->get('description'));
					
				if ($update == 1) {
					$_SESSION['message'][] = ['success', $this->lang['details_updated']];
					redirect('admin/pages/privacy');
				} else {
					$_SESSION['message'][] = ['warning', $this->lang['no_changes_made']];
					redirect('admin/pages/privacy');
				}
					
			 } else {

				 foreach ($validation->errors()->all() as $err) {
					$str = implode(" ",$err);
					 foreach ($err as $r) {
						$_SESSION['errors'][] = ['error', $r];
					 }	
				 }
				 
					redirect('admin/pages/privacy');
		   }

		 }
		}
		

        return ['content' => $this->view->render($data, 'admin/pages')];
    }	
	
	//Random String
	private function rando($length = 14){
		$str = "";
		$characters = array_merge(range('A','Z'), range('a','z'), range('0','9'));
		$max = count($characters) - 1;
		for ($i = 0; $i < $length; $i++) {
			$rand = mt_rand(0, $max);
			$str .= $characters[$rand];
		}
		return $str;
	}
	
	//Random String
	function uniqueid()
	{
		$un = substr(number_format(time() * rand(),0,'',''),0,12);
		return $un;
	}

		/**
	 * Return the slug of a string to be used in a URL.
	 *
	 * @return String
	 */
	private function slugify($text){
		// replace non letter or digits by -
		$text = preg_replace('~[^\pL\d]+~u', '-', $text);

		// transliterate
		$text = iconv('utf-8', 'us-ascii//TRANSLIT', $text);

		// remove unwanted characters
		$text = preg_replace('~[^-\w]+~', '', $text);

		// trim
		$text = trim($text, '-');

		// remove duplicated - symbols
		$text = preg_replace('~-+~', '-', $text);

		// lowercase
		$text = strtolower($text);

		if (empty($text)) {
		  return 'n-a';
		}

		return $text;
	}
	
}